import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import PropTypes from 'prop-types';
import cx from 'classnames';
import { ReactSortable } from 'react-sortablejs';
import ThumbCard from './ThumbCard';
import SkeletonCard from './SkeletonCard';
import { ATOM_ICON_SIZES } from '@s-ui/react-atom-icon';
import { formatToBase64, cropAndRotateImage, base64ToBlob } from './photoTools';
import { callbackUploadPhotoHandler } from './fileTools';
import { BASE_CLASS_NAME, THUMB_CLASS_NAME, THUMB_SORTABLE_CLASS_NAME, DEFAULT_NOTIFICATION_ERROR, DEFAULT_HAS_ERRORS_STATUS, DEFAULT_IMAGE_ROTATION_DEGREES, ROTATION_DIRECTION, DEFAULT_IMAGE_ASPECT_RATIO, DEFAULT_MAX_IMAGE_HEIGHT, DEFAULT_MAX_IMAGE_WIDTH, ACTIONS } from './config';
import { jsx as _jsx } from "react/jsx-runtime";
import { Fragment as _Fragment } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
var PREVIEW_CARD_CLASS_NAME = BASE_CLASS_NAME + "-preview";

var PhotosPreview = function PhotosPreview(_ref) {
  var _cx, _cx2;

  var _callbackPhotosUploaded = _ref._callbackPhotosUploaded,
      _scrollToBottom = _ref._scrollToBottom,
      addMorePhotosIcon = _ref.addMorePhotosIcon,
      addPhotoTextSkeleton = _ref.addPhotoTextSkeleton,
      callbackUploadPhoto = _ref.callbackUploadPhoto,
      defaultFormatToBase64Options = _ref.defaultFormatToBase64Options,
      deleteIcon = _ref.deleteIcon,
      dragDelay = _ref.dragDelay,
      errorInitialPhotoDownloadErrorText = _ref.errorInitialPhotoDownloadErrorText,
      files = _ref.files,
      isPhotoUploaderFully = _ref.isPhotoUploaderFully,
      mainPhotoLabel = _ref.mainPhotoLabel,
      outputImageAspectRatioDisabled = _ref.outputImageAspectRatioDisabled,
      rejectPhotosIcon = _ref.rejectPhotosIcon,
      rotateIcon = _ref.rotateIcon,
      rotationDirection = _ref.rotationDirection,
      retryIcon = _ref.retryIcon,
      setFiles = _ref.setFiles,
      setIsLoading = _ref.setIsLoading,
      setNotificationError = _ref.setNotificationError,
      thumbIconSize = _ref.thumbIconSize;

  var _onSortEnd = function _onSortEnd(event) {
    _callbackPhotosUploaded(files, {
      action: ACTIONS.SORT,
      data: event
    });
  };

  var _deleteItem = function _deleteItem(index) {
    var list = [].concat(files);
    list.splice(index, 1);
    setFiles(list);
    setNotificationError(DEFAULT_NOTIFICATION_ERROR);

    _callbackPhotosUploaded(list, {
      action: ACTIONS.DELETE,
      data: {
        itemIndex: index,
        item: files[index]
      }
    });
  };

  var _rotateItem = function _rotateItem(index) {
    var list = [].concat(files);

    if (rotationDirection === ROTATION_DIRECTION.clockwise) {
      list[index].rotation = list[index].rotation === 270 ? 0 : list[index].rotation + 90;
    } else {
      list[index].rotation = list[index].rotation === 0 ? 270 : list[index].rotation - 90;
    }

    cropAndRotateImage({
      base64Image: list[index].originalBase64,
      rotation: list[index].rotation,
      outputImageAspectRatio: DEFAULT_IMAGE_ASPECT_RATIO,
      maxImageHeight: DEFAULT_MAX_IMAGE_HEIGHT,
      maxImageWidth: DEFAULT_MAX_IMAGE_WIDTH,
      outputImageAspectRatioDisabled: outputImageAspectRatioDisabled
    }).then(function (value) {
      return base64ToBlob(value);
    }).then( /*#__PURE__*/function () {
      var _ref3 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(_ref2) {
        var blob, base64;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                blob = _ref2.blob, base64 = _ref2.base64;
                list[index].preview = base64;
                list[index].blob = blob;
                list[index].isModified = true;
                _context.next = 6;
                return callbackUploadPhotoHandler(blob, callbackUploadPhoto, list[index].url);

              case 6:
                list[index].url = _context.sent;
                setFiles(list);
                setNotificationError(DEFAULT_NOTIFICATION_ERROR);

                _callbackPhotosUploaded(list, {
                  action: ACTIONS.ROTATE,
                  data: {
                    itemIndex: index,
                    item: files[index]
                  }
                });

              case 10:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x) {
        return _ref3.apply(this, arguments);
      };
    }());
  };

  var _retryUpload = function _retryUpload(index) {
    setIsLoading(true);

    var _files = [].concat(files);

    var photoToRetry = _files[index];
    formatToBase64({
      item: photoToRetry,
      options: defaultFormatToBase64Options
    }).then(function (_ref4) {
      var blob = _ref4.blob,
          croppedBase64 = _ref4.croppedBase64,
          url = _ref4.url,
          _ref4$hasErrors = _ref4.hasErrors,
          hasErrors = _ref4$hasErrors === void 0 ? DEFAULT_HAS_ERRORS_STATUS : _ref4$hasErrors;

      if (hasErrors) {
        setNotificationError({
          isError: true,
          text: errorInitialPhotoDownloadErrorText
        });

        _scrollToBottom();
      } else {
        setNotificationError(DEFAULT_NOTIFICATION_ERROR);
      }

      _files[index] = {
        blob: blob,
        url: url,
        hasErrors: hasErrors,
        originalBase64: croppedBase64,
        preview: croppedBase64,
        rotation: DEFAULT_IMAGE_ROTATION_DEGREES,
        isNew: false,
        isModified: false
      };
      setFiles([].concat(_files));

      _callbackPhotosUploaded(_files, {
        ACTION: ACTIONS.RETRY_UPLOAD,
        data: {
          itemIndex: index,
          item: files[index]
        }
      });

      setIsLoading(false);
    });
  };

  var thumbClassName = cx(THUMB_CLASS_NAME, (_cx = {}, _cx[THUMB_SORTABLE_CLASS_NAME] = files.length > 1, _cx));

  var thumbCards = function thumbCards(files) {
    return files.map(function (file, index) {
      return /*#__PURE__*/_jsx("li", {
        className: thumbClassName,
        onClick: function onClick(e) {
          return e.stopPropagation();
        },
        children: /*#__PURE__*/_jsx(ThumbCard, {
          iconSize: thumbIconSize,
          image: file,
          index: index,
          mainPhotoLabel: mainPhotoLabel,
          callbackDeleteItem: _deleteItem,
          callbackRetryUpload: _retryUpload,
          callbackRotateItem: _rotateItem,
          rotateIcon: rotateIcon(),
          deleteIcon: deleteIcon(),
          retryIcon: retryIcon(),
          rejectPhotosIcon: rejectPhotosIcon(),
          outputImageAspectRatioDisabled: outputImageAspectRatioDisabled
        })
      }, "" + file.preview + index);
    });
  };

  var previewCardClass = cx(PREVIEW_CARD_CLASS_NAME, (_cx2 = {}, _cx2[PREVIEW_CARD_CLASS_NAME + "--ratioDisabled"] = outputImageAspectRatioDisabled, _cx2));
  return /*#__PURE__*/_jsx(ReactSortable, {
    className: previewCardClass,
    handle: ".sui-MoleculePhotoUploader-thumbCard-imageContainer",
    ghostClass: THUMB_CLASS_NAME + "--ghost",
    dragClass: THUMB_CLASS_NAME + "--drag",
    chosenClass: THUMB_CLASS_NAME + "--chosen",
    tag: "ul",
    list: files,
    setList: setFiles,
    animation: 200,
    draggable: "." + THUMB_SORTABLE_CLASS_NAME,
    onEnd: function onEnd(event) {
      return _onSortEnd(event);
    },
    delay: dragDelay,
    children: /*#__PURE__*/_jsxs(_Fragment, {
      children: [thumbCards(files), !isPhotoUploaderFully() && /*#__PURE__*/_jsx(SkeletonCard, {
        icon: addMorePhotosIcon(),
        text: addPhotoTextSkeleton
      })]
    })
  });
};

PhotosPreview.displayName = 'PhotosPreview';
export default PhotosPreview;